package cn.efunbox.audio.impl.hag;

import cn.efunbox.audio.entity.hag.HagProduct;
import cn.efunbox.audio.repository.hag.HagProductRepository;
import cn.efunbox.audio.service.hag.HagProductService;
import cn.efunbox.audio.utils.DateUtil;
import cn.efunbox.audio.vo.hag.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * HagProductServiceImpl
 * Created by xusq on 2019/12/25.
 */
@Service
public class HagProductServiceImpl implements HagProductService {

    @Autowired
    private HagProductRepository hagProductRepository;

    @Override
    public HagProductResp list(HagProductReq productReq) {

        Pagination paginationReq = productReq.getPagination();
        Integer limit = paginationReq.getLimit();
        if (Objects.isNull(limit)) {
            limit = 10;
        }
        Integer start = paginationReq.getStart();
        if (Objects.isNull(start)) {
            start = 0;
        }
        HagProductResp hagProductResp = new HagProductResp();
        String updateTime = productReq.getUpdateTime();
        Date date = DateUtil.strToDate(updateTime);
        if (Objects.isNull(date)) {
            hagProductResp.setErrorCode("400");
            hagProductResp.setErrorMessage("timestamp数据异常");
            return hagProductResp;
        }


        long count = hagProductRepository.countByGmtModifiedAfter(date);

        Pagination pagination = new Pagination();
        pagination.setTotal(count);

        if (count == 0) {
            pagination.setNext("0");
            hagProductResp.setPagination(pagination);
            hagProductResp.setContentItems(new ArrayList<>());
            return hagProductResp;
        }

        pagination.setNext(start + limit + "");

        List<HagProduct> hagProducts = hagProductRepository.findByGmtModifiedAfterPage(date,start,limit);



        List<String> albumIds = new ArrayList<>();

        hagProducts.forEach(product -> albumIds.add(product.getAlbumId()));

        List<ProductVO> productVOList = new ArrayList<>();

        if (!CollectionUtils.isEmpty(hagProducts)) {
            hagProducts.forEach(product ->
                productVOList.add(ProductConverter.productToProductVO(product))
            );
        }
        hagProductResp.setPagination(pagination);
        hagProductResp.setContentItems(productVOList);

        return hagProductResp;
    }
}
