package cn.efunbox.audio.utils;

import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.lang.StringUtils;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.io.UnsupportedEncodingException;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.*;

/**
 * 常見的几种加密方式
 */
@Slf4j
public class Encrypt {

    public static final String SIGN = "sign";

    private static Base64 base64;

    static {
        base64 = new Base64();
    }


    public static String createSHA256Sign(Map<String, String> paramMap, String signKey) {

        List<String> sortedKeys = new ArrayList<String>();
        for (Map.Entry<String, String> entry : paramMap.entrySet()) {
            if (SIGN.equals(entry.getKey())) {
                continue;
            }

            sortedKeys.add(entry.getKey());
        }

        if (sortedKeys.size() == 0) {
            // 没有参数
            return "";
        }

        Collections.sort(sortedKeys);

        StringBuffer buff = new StringBuffer("");
        for (String key : sortedKeys) {
            String val = paramMap.get(key);
            if (StringUtils.isBlank(val)) {
                continue;
            }

            buff.append(key).append("=").append(val).append("&");
        }

        buff.append("key=").append(signKey);

        try {
            return Encrypt.String2SHA256(buff.toString());
        } catch (Exception e) {
            throw new RuntimeException("签名错误");
        }
    }

    /**
     *  利用Apache的工具类实现SHA-256加密
     * @param str 加密后的报文
     * @return
     */
    public static String String2SHA256(String str){
        MessageDigest messageDigest;
        String encdeStr = "";
        try {
            messageDigest = MessageDigest.getInstance("SHA-256");
            byte[] hash = messageDigest.digest(str.getBytes("UTF-8"));
            encdeStr = Hex.encodeHexString(hash);
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return encdeStr;
    }
    //---------
    /**
     * 利用java原生的摘要实现SHA256加密
     * @param str 加密后的报文
     * @return
     */
    public static String String2SHA256StrJava(String str){
        MessageDigest messageDigest;
        String encodeStr = "";
        try {
            messageDigest = MessageDigest.getInstance("SHA-256");
            messageDigest.update(str.getBytes("UTF-8"));
            encodeStr = byte2Hex(messageDigest.digest());
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return encodeStr;
    }
 
    /**
     * 将byte转为16进制
     * @param bytes
     * @return
     */
    private static String byte2Hex(byte[] bytes){
        StringBuffer stringBuffer = new StringBuffer();
        String temp = null;
        for (int i=0;i<bytes.length;i++){
            temp = Integer.toHexString(bytes[i] & 0xFF);
            if (temp.length()==1){
                //1得到一位的进行补0操作
                stringBuffer.append("0");
            }
            stringBuffer.append(temp);
        }
        return stringBuffer.toString();
    }

    public static String createHMACSHA256(Map<String, String> paramMap, String signKey) {

        List<String> sortedKeys = new ArrayList<>();
        for (Map.Entry<String, String> entry : paramMap.entrySet()) {
            if (SIGN.equals(entry.getKey())) {
                continue;
            }

            sortedKeys.add(entry.getKey());
        }

        if (sortedKeys.size() == 0) {
            // 没有参数
            return "";
        }

        Collections.sort(sortedKeys);

        StringBuffer buff = new StringBuffer("");
        for (String key : sortedKeys) {
            String val = paramMap.get(key);
            if (StringUtils.isBlank(val)) {
                continue;
            }

            buff.append(key).append("=").append(val).append("&");
        }

        buff.deleteCharAt(buff.length() - 1);

        try {
            return HMACSHA256(buff.toString(),signKey);
        } catch (Exception e) {
            throw new RuntimeException("签名错误");
        }
    }

    public static String HMACSHA256(String data, String key) throws Exception {

        Mac sha256_HMAC = Mac.getInstance("HmacSHA256");

        SecretKeySpec secret_key = new SecretKeySpec(key.getBytes("UTF-8"), "HmacSHA256");

        sha256_HMAC.init(secret_key);

        byte[] array = sha256_HMAC.doFinal(data.getBytes("UTF-8"));

        String encodeSign = base64.encodeToString(array);

        return encodeSign;

    }


    public static void main(String[] args) {

        //idChannel=100103&idDevice=10.31.49.16&ts=159559185715
        Map<String,String> param = new HashMap<>();
        param.put("idChannel","100103");
        param.put("idDevice","10.31.49.16");
        param.put("x-ts","159559185715");
        log.info(JSONObject.toJSONString(param));
        String sign = Encrypt.createHMACSHA256(param,"IhOTiTyMLDNNLFuP");
        log.info(sign);

    }
}
